//
// Epsplitter.cc
//
// Authors:  Richard Saxton (VEGA/ESA)
//           ESA XMM SOC
//
// Description: Task to produce a fluxed spectrum
//
//#include <ctime>

#include "CalServer.h"
#include "Dal.h"
#include "OalServer.h"
#include "FileLocator.h"
#include "DataFilter.h"

#include "Epsplitter.h"

#include "Tracer.h"
#include "errstr.h"
#include "param.h"
#include "slatec.h"
#include <DirEntry.h>
#include <DirEntryFactory.h>

/* Method: Epsplitter() - Constructor */
Epsplitter::Epsplitter()
{
  PROC;


  /* Get the parameters from this command line */
    string inFile = stringParameter("pnfile");
    int slices = intParameter("slices");
    //dataSetServer->copy(inFile,"ori.ds");
    const Odf *odf = oalServer.odf();

    errstr << "Processing ODF Directory... " << odf->directoryName()
		<< Sas::message(Msg::App,Msg::Sparse);

    //FileLocator myfileLocator(odf->directoryName());
    //string myFile = myfileLocator.locate(inFile);

    errstr <<"Splitting... " << inFile
		<< Sas::message(Msg::App,Msg::Sparse);

    DirEntry* d = dirEntryFactory->dirEntry(inFile);
    oalServer.setState(inFile);

	Xmm::DataMode dataMode =  oalServer.dataMode();

    string tableName;

    if (dataMode == Xmm::IMAGING)
    	tableName = "PNIME1";
    else if (dataMode == Xmm::TIMING)
    	tableName = "PNTIE1";
    else if (dataMode == Xmm::BURST)
		tableName = "PNBUE1";
    else
      errstr << "Data Mode: "<< dataMode << " not found"<< Sas::error("DataModeNotFound");
    
    
    DataSet *ds = dataSetServer->open(inFile,Dal::Read);

    Table *tb = ds->table(tableName);
    int chop =(tb->rows()-1)/slices;
    dataSetServer->close(ds);

    int chopper = 0;
    int rowBegin = 0;
    int rowEnd = 0;
    //Loop over the slices
    for (int i = 1 ; i <= slices ; ++i)
    {

      // Original Dataset needed to create the expression
      DataSet *ds  = dataSetServer->open(inFile,DataSet::Read);
      Table *tb = ds->table(tableName);
      Column *col = tb->column("FRAME");
      ColumnData *frameDat = col->data();     
      int32 *frames = frameDat->int32Data();

      //place to cut
      chopper = chop+chopper;
      stringstream expression;      
      if (i == 1)
	{
	  rowBegin = frames[chopper];
	  expression << "(FRAME <= "<<rowBegin<<")";
	}
      else if (i == slices)
	{
	  expression << "(FRAME > "<<rowBegin<<")";	  
	}
      else
	{
	  rowEnd = frames[chopper];
	  expression << "(FRAME in ("<<rowBegin <<":"<<rowEnd<<"])";
	  rowBegin = rowEnd;	  
	}
      delete frameDat;
      dataSetServer->close(ds); 

      // New Dataset
      string part = d->basename();
      stringstream suffix ;
      suffix << "_part"<<i<<".FIT";
      part.replace(part.find_first_of("."),4,suffix.str());
      

      errstr << "Copying input file to...  " << part
	     << Sas::message(Msg::App,Msg::Sparse);
      DataSet *clone;
      dataSetServer->copy(inFile,part);
      clone = dataSetServer->open(part,DataSet::Modify);
      Table *tb1 = clone->table(tableName);



      errstr << "Filtering "<< part <<"... " << Sas::message(Msg::App,Msg::Sparse);
      errstr << "Expression " << expression.str() << Sas::message(Msg::App,Msg::Sparse);
      
      DataFilter filter1(expression.str());
      unsigned long selectedEvents = filter1.doFilter(*tb1);
      errstr << "selected " << selectedEvents << " rows from part1."
	     << Sas::message(Msg::App,Msg::Sparse);
      
      errstr << "Closing data set "<< clone->name()<< " ..." << Sas::message(Msg::App,Msg::Sparse);

      dataSetServer->close(clone);    
    }





ENDPROC;
}

/* Destructor */
Epsplitter:: ~Epsplitter()
{
  PROC;

  ENDPROC;
}

